-- ----------------------------------------------------------------------------
-- Product: SAS Drug Development
-- Database: Oracle
-- Description: DDL for privilege services
-- Copyright: 2011, SAS Institute Inc., Cary, NC
-- ----------------------------------------------------------------------------
-- $Id$
-- $DateTime$
-- $Change$
-- $Author$
-- ----------------------------------------------------------------------------

create table sas_privilege (
  id varchar2(36) not null,
  version number(10) default -1 not null,
  object_type varchar2(128) not null,
  object_identifier varchar2(36) not null,
  privilege_id varchar2(64) not null,
  constraint pk_sas_privilege primary key (id) using index tablespace users,
  constraint uk_sas_privilege unique (object_type, object_identifier, privilege_id) using index tablespace users
) tablespace users;

create table sas_privilege_member (
  privilege_assignment_id varchar2(36) not null,
  principal_type varchar2(128) not null,
  principal_identifier varchar2(36) not null,
  constraint pk_sas_privilege_member primary key (privilege_assignment_id, principal_type, principal_identifier) using index tablespace users,
  constraint fk_sas_privilege_member foreign key (privilege_assignment_id) references sas_privilege(id) on delete cascade
) tablespace users;

create table sas_role (
  id varchar2(36) not null,
  version number(10) default -1 not null,
  object_type varchar2(128) not null,
  object_identifier varchar2(36) not null,
  name nvarchar2(64) not null,
  description nvarchar2(256),
  constraint pk_sas_role primary key (id) using index tablespace users,
  constraint uk_sas_role unique (object_type, object_identifier, name) using index tablespace users
) tablespace users;

create table sas_role_privilege (
  role_id varchar2(36) not null,
  privilege_id varchar2(64) not null,
  constraint pk_sas_role_privilege primary key (role_id, privilege_id) using index tablespace users,
  constraint fk_sas_role_privilege foreign key (role_id) references sas_role(id) on delete cascade
) tablespace users;

create table sas_role_assignment (
  id varchar2(36) not null,
  version number(10) default -1 not null,
  object_type varchar2(128) not null,
  object_identifier varchar2(36) not null,
  role_id varchar2(36) not null,
  constraint pk_sas_role_assignment primary key (id) using index tablespace users,
  constraint uk_sas_role_assignment unique (object_type, object_identifier, role_id) using index tablespace users,
  constraint fk_sas_role_assignment foreign key (role_id) references sas_role(id) on delete cascade
) tablespace users;

create table sas_role_assignment_member (
  role_assignment_id varchar2(36) not null,
  principal_type varchar2(128) not null,
  principal_identifier varchar2(36) not null,
  constraint pk_sas_role_assignment_member primary key (role_assignment_id, principal_type, principal_identifier) using index tablespace users,
  constraint fk_sas_role_assignment_member foreign key (role_assignment_id) references sas_role_assignment(id) on delete cascade
) tablespace users;
