-- ----------------------------------------------------------------------------
-- Product: SAS Drug Development
-- Database: Oracle
-- Description: DDL for user services
-- Copyright: 2011, SAS Institute Inc., Cary, NC
-- ----------------------------------------------------------------------------
-- $Id$
-- $DateTime$
-- $Change$
-- $Author$
-- ----------------------------------------------------------------------------
	
create table sas_attributes (
  objecttype varchar2(32) not null,
  objectid varchar2(36) not null,
  attributename nvarchar2(64) not null,
  attributetype varchar2(1) not null,
  booleanvalue number(1),
  datevalue number(13),
  longvalue number(13),
  stringvalue nvarchar2(2000),
  constraint pk_sas_attributes primary key (objecttype, objectid, attributename) using index tablespace users
) tablespace users;

create table sas_users (
  id varchar2(36) not null,
  revision number(10) default 1 not null,
  userid varchar2(32) not null,
  firstname nvarchar2(32),
  lastname nvarchar2(32),
  displayname nvarchar2(128) not null,
  department nvarchar2(256),
  email varchar2(256) not null,
  phone varchar2(32),
  accounttype varchar2(32) not null,
  licensetype varchar2(32) not null,
  password varchar2(64),
  passworddate number(13),
  activationdate number(13),
  deactivationdate number(13),
  loginattempts number(3) default 0 not null,
  logincount number(10) default 0 not null,
  lastlogin number(13),
  active number(1) default 0 not null,
  statusby varchar2(32),
  statusdate number(13),
  locked number(1) default 0 not null,
  lockedby varchar2(32),
  lockeddate number(13),
  uidnumber number(5),
  gidnumber number(5) default 1000,
  constraint pk_sas_user primary key (id) using index tablespace users,
  constraint uk_sas_user_userid unique (userid) using index tablespace users
) tablespace users;

create table sas_user_passwords (
  userid varchar2(32),
  password varchar2(64),
  created number(13),
  constraint pk_user_passwords primary key (userid, password) using index tablespace users,
  constraint fk_user_passwords foreign key (userid) references sas_users(userid) on delete cascade
) tablespace users;

-- ----------------------------------------------------------------------------
-- Sequence and trigger for handling OS UID
-- ----------------------------------------------------------------------------
create sequence seq_sas_user_uid
	increment by 1
	maxvalue 4294967295 
	start with 10000
	cache 20
	noorder
	nocycle;


-- ----------------------------------------------------------------------------
-- Default insert statements
-- ----------------------------------------------------------------------------
insert into sas_users(id, userid, displayname, email, password, accounttype, licensetype, active) 
  values('1', 'sasinst', 'SAS Installer Account', 'noreply@sas.com', 'Gfttm3S2x1DRvQh8BtwM0w==', 'INTERACTIVE', 'administrator', 1);
